﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.XR.iOS;

public class LaserScript : MonoBehaviour {

	public Transform startPoint;
	public Transform endPoint;
	public Camera mainCamera;
	public bool debugLaser;

	LineRenderer laserLine; 
	ParticleSystem tranceParticles;
	AudioSource music;

	bool shapeEnabled = false;
	Dictionary<string, float> currentBlendShapes;


	// Use this for initialization
	void Start () {
		laserLine = GetComponent<LineRenderer> (); 
		laserLine.startWidth = 0.2f;
		laserLine.endWidth = 0.2f;

		tranceParticles = startPoint.GetComponent<ParticleSystem> ();

		music = mainCamera.GetComponent<AudioSource> ();

		UnityARSessionNativeInterface.ARFaceAnchorAddedEvent += FaceAdded;
		UnityARSessionNativeInterface.ARFaceAnchorUpdatedEvent += FaceUpdated;
		UnityARSessionNativeInterface.ARFaceAnchorRemovedEvent += FaceRemoved;

		music.volume = 0.0f;
		tranceParticles.Stop();
	
	}

	// Update is called once per frame
	void Update () {
		laserLine.SetPosition (0, startPoint.position);
		laserLine.SetPosition (1 , endPoint.position );
		var sh = tranceParticles.shape;

		laserLine.enabled = false;

		if (shapeEnabled) {

			int jaw = (int)(currentBlendShapes ["jawOpen"] * 10);
			int mouthpucker = (int)(currentBlendShapes ["mouthPucker"] * 10);

			Debug.Log ("jaw: " + jaw);

			if (jaw > 1 && mouthpucker<3) {
				sh.angle = jaw / 2;
				sh.radius = jaw / 2;
				laserLine.startWidth = (float)jaw / 30;
				laserLine.endWidth = (float)jaw / 20;
		//		laserLine.enabled = true;
				tranceParticles.Play ();
				music.volume = (float)jaw/10;
			
			} else {
				tranceParticles.Stop();
				music.volume = 0.0f;
			}
		} else {
			Debug.Log ("no jaw :(");
			tranceParticles.Stop();
			music.volume = 0.0f;
		}

		if (debugLaser) {
			int jawtest = 3;
			sh.angle = jawtest / 2;
			sh.radius = jawtest / 2;
			laserLine.startWidth = (float)jawtest / 30;
			laserLine.endWidth = (float)jawtest / 20;
			//	laserLine.enabled = true;
			tranceParticles.Play ();
		}
	}

	void FaceAdded (ARFaceAnchor anchorData)
	{
		shapeEnabled = true;
		currentBlendShapes = anchorData.blendShapes;
	}

	void FaceUpdated (ARFaceAnchor anchorData)
	{
		currentBlendShapes = anchorData.blendShapes;
	}

	void FaceRemoved (ARFaceAnchor anchorData)
	{
		shapeEnabled = false;
	}


}
